package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NitpamStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NitpamStoreVo;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Nitpam Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NitpamStructMapper extends IBaseStructMapper<NitpamStoreBo, NitpamStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NitpamStructMapper INSTANCE = Mappers.getMapper(NitpamStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NitpamStoreBo voTransferToBo(NitpamStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldnidgrp",ignore = true),
            //修改交易，两个机构记账机构和所属机构忽略赋值
            @Mapping(target = "nidgrp.rec.branchinr",ignore = true),
            @Mapping(target = "nidgrp.rec.bchkeyinr",ignore = true)
    })
    void voUpdateToBo(NitpamStoreVo vo, @MappingTarget NitpamStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NitpamStoreVo boTransferToVo(NitpamStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NitpamStoreBo bo, @MappingTarget NitpamStoreVo vo);
}
